// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "Components/ActorComponent.h"
#include "BuoyancyComponent.generated.h"

USTRUCT()
struct FVertexData
{
	GENERATED_BODY()

	FVector Position;
	// depth of the vertex above the surface of the water
	// negative value means the vertex is submerged
	float Depth;

	int32 Index;

	FVertexData() {}

	FVertexData(FVector Position)
	{
		this->Position = Position;
		Depth = 0.0f;
	}

	FVertexData(FVector Position, float Depth)
	{
		this->Position = Position;
		this->Depth = Depth;
	}
	
	void CalculateDepth(float SurfaceHeight)
	{
		Depth = Position.Z - SurfaceHeight;
	}

	bool operator< (const FVertexData& rhs) const
	{
		return this->Depth < rhs.Depth;
	}
};

USTRUCT()
struct FTriangleData
{
	GENERATED_BODY()

	FVertexData Highest;
	FVertexData Middle;
	FVertexData Lowest;

	FTriangleData() {}
	
	FTriangleData(FVertexData V1, FVertexData V2, FVertexData V3)
	{
		TArray<FVertexData> Vertices = {V1, V2, V3};
		Vertices.Sort();
		Lowest = Vertices[0];
		Middle = Vertices[1];
		Highest = Vertices[2];
	}

	int32 GetNumSubmergedVertices()
	{
		int32 result = 0;
		if(Highest.Depth < 0.0f)
		{
			result++;
		}
		if(Middle.Depth < 0.0f)
		{
			result++;
		}
		if(Lowest.Depth < 0.0f)
		{
			result++;
		}
		return result;
	}
};

UCLASS( ClassGroup=(Custom), meta=(BlueprintSpawnableComponent) )
class SEAOFCROOKS_API UBuoyancyComponent : public UActorComponent
{
	GENERATED_BODY()

public:	
	// Sets default values for this component's properties
	UBuoyancyComponent();

protected:
	UPROPERTY(BlueprintReadOnly)
	UStaticMesh* VolumeMesh;

	UPROPERTY()
	TArray<FTriangleData> SubmergedTriangles;
	
	// Called when the game starts
	virtual void BeginPlay() override;

public:	
	// Called every frame
	virtual void TickComponent(float DeltaTime, ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction) override;

private:
	int32 GetNumTriangles();
	
	TArray<FVertexData> GetVertexPositions();

	TArray<int32> GetTriangleIndices();

	void ProcessTriangles(TArray<FTriangleData>& Triangles);
};
